package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.sms.entity.Role;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.RoleService;
import com.bocloud.sms.model.ApiInfo;
import com.bocloud.sms.model.CloudServiceBean;
import com.bocloud.sms.model.RoleModel;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/roles")
@Tag(name = "角色管理")
public class RoleController {

    private final RoleService roleService;

    public RoleController(RoleService roleService) {
        this.roleService = roleService;
    }

    /**
     * 添加角色
     *
     * @param roleModel      角色
     * @param requestContext 上下文
     * @return 添加结果
     */
    @Operation(summary = "创建角色")
    @PostMapping
    public Result create(@RequestBody RoleModel roleModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.create(roleModel, requestContext);
    }

    /**
     * 修改角色
     *
     * @param roleModel      角色
     * @param requestContext 上下文
     * @return 修改结果
     */
    @Operation(summary = "角色修改")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody RoleModel roleModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.modify(id, roleModel, requestContext);
    }

    /**
     * 移除角色
     *
     * @param id             角色id
     * @param requestContext 上下文
     * @return 移除结果
     */
    @Operation(summary = "角色删除")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.remove(id, requestContext);
    }

    /**
     * 查看角色详细信息
     *
     * @param id 角色id
     * @return 角色
     */
    @Operation(summary = "角色详情")
    @GetMapping("/{id}")
    public GeneralResult<Role> detail(@PathVariable(value = Common.ID) Long id) {
        return roleService.detail(id);
    }

    /**
     * 角色授权
     *
     * @param id             角色id
     * @param ids            用户id
     * @param requestContext 上下文
     * @return 授权结果
     */
    @Operation(summary = "角色授权")
    @PatchMapping("/{id}")
    public Result accredit(@PathVariable(value = Common.ID) Long id, @RequestBody List<Long> ids,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.accredit(id, ids, requestContext);
    }

    /**
     * 查询角色的权限列表
     *
     * @param id             角色id
     * @param requestContext 上下文
     * @return 权限列表
     */
    @Operation(summary = "获取权限列表")
    @GetMapping("/{id}/permissions")
    public GeneralResult<List<CloudServiceBean>> permissions(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "menu", required = false) boolean menu,
                                                             @RequestParam(value = "tenant", required = false) boolean tenant, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.listPermissions(id, menu, tenant, requestContext);
    }

    /**
     * 查询角色列表
     *
     * @param pager          分页参数
     * @param requestContext 上下文
     * @return 角色列表
     */
    @Operation(summary = "获取角色列表")
    @GetMapping
    public GeneralResult<GridBean<Role>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.list(pager, requestContext);
    }

    /**
     * 获取请求用户角色信息（用于boot comment 获取角色Api权限）
     *
     * @param requestContext 请求用户
     * @return 角色信息
     */
    @GetMapping("/ids")
    public GeneralResult<List<Long>> roleIds(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return roleService.roleIds(requestContext);
    }

    /**
     * API授权
     *
     * @param id   角色id
     * @param apis apiid列表
     * @return 授权结果
     */
    @Operation(summary = "API授权")
    @PatchMapping("/{id}/apis")
    public Result accreditRole(@PathVariable(value = Common.ID) Long id, @RequestBody List<ApiInfo> apis) {
        return roleService.accreditRole(id, apis);
    }

    /**
     * 查询API权限列表
     *
     * @param id   角色id
     * @param tags
     * @return 权限列表
     */
    @Operation(summary = "查询API权限列表")
    @GetMapping("/{id}/apis")
    public GeneralResult<JSONArray> getModuleList(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "tags", required = false) String tags) {
        return roleService.getApiPermission(id, tags);
    }

    /**
     * 查询指定角色下的用户
     *
     * @param pager 分页参数
     * @param id    角色id
     * @return 用户列表
     */
    @Operation(summary = "查询指定角色下的用户")
    @GetMapping("/{id}/users")
    public GeneralResult<GridBean<User>> listUsersByRoleId(Pager pager, @PathVariable(value = Common.ID) Long id) {
        return roleService.listUsersByRoleId(pager, id);
    }
}
