package com.bocloud.sms.service;

import com.bocloud.sms.entity.User;
import com.bocloud.sms.entity.UserGroup;
import com.bocloud.sms.entity.UserGroupRelation;
import com.bocloud.sms.interfaces.UserGroupService;
import com.bocloud.sms.model.UserGroupBean;
import com.bocloud.sms.repository.UserGroupRelationRepository;
import com.bocloud.sms.repository.UserGroupRepository;
import com.bocloud.sms.repository.UserRepository;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 用户组抽象Service接口实现类
 *
 * @author hongqingyu
 */
@Service("userGroupService")
@Slf4j
public class UserGroupServiceImpl implements UserGroupService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private UserGroupRelationRepository userGroupRelationRepository;
    @Autowired
    private UserGroupRepository userGroupRepository;
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public Result create(UserGroup group, RequestContext context) {
        try {
            if (StringUtils.isBlank(group.getName())) {
                log.error("Create user group fail: missing name");
                return new Result(false, "添加用户组失败：缺少必要参数");
            }
            if (checkNameExist(group.getName(), group.getId())) {
                log.error("Create user group fail: name is exist");
                return new Result(false, "添加用户组失败：名称已存在");
            }
            group.setCreatorId(getTarget(context));
            group.setMenderId(getTarget(context));
            group.setGmtCreate(new Date());
            group.setGmtModify(new Date());
            group.setDeleted(false);
            userGroupRepository.save(group);
            if (null != group.getUserIds() && group.getUserIds().size() > 0) {
                configUser(group.getUserIds(), group.getId());
            }
            redisTemplate.opsForValue().set("Group_Name_" + group.getId(), group.getName());
            return new Result(true, "添加用户组成功");
        } catch (Exception e) {
            log.error("Create user group fail:", e);
            return new GeneralResult<>(false, "新增用户组失败!");
        }
    }

    @Override
    public Result remove(Long id, RequestContext context) {
        String path = UserGroup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            UserGroup userGroup = userGroupRepository.query(id);
            if (null == userGroup) {
                log.error("Remove user group fail: data is not exist");
                return new Result(false, "删除用户组失败：数据不存在");
            }
            userGroupRepository.delete(id, getTarget(context));
            // 用户组删除后需要删除与用户的关系
            userGroupRelationRepository.deleteByGroupId(id);
            redisTemplate.delete("Group_Name_" + id);
            return new Result(true, "删除用户组成功");
        } catch (Exception e) {
            log.error("Remove user group fail:", e);
            return new GeneralResult<>(false, "删除用户组失败!");
        }
    }

    @Override
    public Result modify(UserGroupBean bean, RequestContext context) {
        String path = UserGroup.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (StringUtils.isBlank(bean.getName())) {
                log.error("Modify user group fail: missing name");
                return new Result(false, "编辑用户组失败：缺少必要参数");
            }

            UserGroup group = userGroupRepository.query(bean.getId());
            if (null == group) {
                log.error("Modify user group fail: data is not exist");
                return new Result(false, "编辑用户组失败：数据不存在");
            }

            if (checkNameExist(bean.getName(), bean.getId())) {
                log.error("Modify user group fail: name is exist");
                return new Result(false, "编辑用户组失败：名称已存在");
            }

            BeanUtils.copyProperties(bean, group);
            group.setMenderId(getTarget(context));
            group.setGmtModify(new Date());
            userGroupRepository.update(group);
            if (null != group.getUserIds() && group.getUserIds().size() > 0) {
                configUser(group.getUserIds(), group.getId());
            } else {
                configUser(new ArrayList<>(), group.getId());
            }
            redisTemplate.opsForValue().set("Group_Name_" + group.getId(), group.getName());
            return new Result(true, "编辑用户组成功");
        } catch (Exception e) {
            log.error("Modify user group failure:", e);
            return new GeneralResult<>(false, "编辑用户组失败!");
        }
    }

    @Override
    public GeneralResult<UserGroup> detail(Long id) {
        try {

            UserGroup group = userGroupRepository.query(id);
            if (null == group) {
                log.error("Query user group fail: data is not exist");
                return new GeneralResult<>(false, "查询用户组详情失败：数据不存在");
            }
            List<Long> users = userGroupRelationRepository.listUserIdByGroupId(group.getId());
            group.setUserIds(users);
            return new GeneralResult<>(true, group, "查询用户组详情成功");
        } catch (Exception e) {
            log.error("Query user group fail: ", e);
            return new GeneralResult<>(false, "查询用户组失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult list(Pager pager) {
        try {
            GridBean<UserGroup> gridBean;
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = userGroupRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<SimpleBean> beans = userGroupRepository.list(pager.getParams(), pager.getSorter());
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                List<UserGroup> list = userGroupRepository.list(pager);
                list.stream().forEach(group -> {
                    group.setUserCount(userRepository.countByGroupId(group.getId()));
                });
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询用户组成功");
        } catch (Exception e) {
            log.error("list user group fail:", e);
            return new GeneralResult<>(false, "查询用户组失败:" + e.getMessage());
        }
    }

    @Override
    public Result configUser(List<Long> userIds, Long id) {
        try {
            UserGroup group = userGroupRepository.query(id);
            if (null == group) {
                log.error("Config user fail: data is not exist");
                return new Result(false, "配置用户失败：用户组不存在");
            }
            List<Long> oldUserIds = userGroupRelationRepository.listUserIdByGroupId(id);
            userIds.stream().forEach(userId -> {
                if (!oldUserIds.contains(userId)) {
                    userGroupRelationRepository.save(new UserGroupRelation(userId, id));
                } else {
                    oldUserIds.remove(userId);
                }
            });
            if (oldUserIds.size() > 0) {
                userGroupRelationRepository.deleteByGroupId(id, oldUserIds);
            }
            return new GeneralResult<>(true, "配置用户成功");
        } catch (Exception e) {
            log.error("config user group fail:", e);
            return new GeneralResult<>(false, "配置用户失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult listGroupIdByUserId(Long userId) {
        try {
            List<UserGroupRelation> groupIds = userGroupRelationRepository.listGroupIdByUserId(userId);
            return new GeneralResult<>(true, groupIds, "查询用户组成功");
        } catch (Exception e) {
            log.error("List user group id fail:", e);
            return new GeneralResult<>(false, "查询用户组失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult listUserByGroupId(Pager pager, Long groupId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
//            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = userRepository.countUser(pager.getParams(), groupId);
            List<User> list = userRepository.listUser(pager, groupId);
            GridBean<User> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询用户成功");
        } catch (Exception e) {
            log.error("list user group fail:", e);
            return new GeneralResult<>(false, "查询用户失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult listAll() {
        try {
            List<UserGroup> list = userGroupRepository.list();
            return new GeneralResult<>(true, list, "查询用户成功");
        } catch (Exception e) {
            log.error("list user group fail:", e);
            return new GeneralResult<>(false, "查询用户组失败:" + e.getMessage());
        }
    }

    /**
     * 判断名称是否存在
     *
     * @param name 名称
     * @param id   当前记录id
     * @return 名称是否存在
     */
    private boolean checkNameExist(String name, Long id) {
        if (userGroupRepository.countName(name, id) > 0) {
            return true;
        }
        return false;
    }

    private Long getTarget(RequestContext context) {
        return null == context ? 1L : context.getTarget();
    }
}
