package com.bocloud.sms.repository;

import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.bocloud.sms.entity.Message;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 消息数据接口实现
 *
 * @author weiwei
 * @version 1.0
 * @since 2016.10.26
 */
@Component("messageRepository")
public class MessageRepository extends JdbcGenericDao<Message, Long> {

    public MessageRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 分页获取消息列表
     *
     * @param page   页码
     * @param rows   行数
     * @param params 查询参数
     * @param sorter 排序参数
     * @return
     * @throws Exception
     */
    public List<Message> list(int page, int rows, List<Param> params, Map<String, String> sorter,
                              RequestContext.Catalog catalog) throws DataBaseException {
        String sql =
                "select a.*,b.name sender_name from message a left join users b on a.sender = b.id where a.is_deleted = 0";
        if (catalog.equals(RequestContext.Catalog.Tenant)) {
            sql += " and catalog = 'Tenant' ";
        } else {
            sql += " and (catalog = 'Manager' or catalog = 'User') ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(Message.class, sql, paramMap);
    }

    /**
     * 获取消息列表
     *
     * @param params 查询参数
     * @param sorter 排序参数
     * @return
     * @throws Exception
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, RequestContext.Catalog catalog)
            throws DataBaseException {
        String sql = "select a.id,a.name from message a where a.is_deleted = 0";
        if (catalog.equals(RequestContext.Catalog.Tenant)) {
            sql += " and catalog = 'Tenant' ";
        } else {
            sql += " and (catalog = 'Manager' or catalog = 'User') ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        List<Message> appList = this.list(Message.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (Message app : appList) {
            beans.add(new SimpleBean(app.getId(), app.getName()));
        }
        return beans;
    }

    /**
     * 删除消息
     *
     * @param id       消息id
     * @param menderId 修改者id
     * @return
     * @throws Exception
     */
    public boolean remove(Long id, Long menderId) throws DataBaseException {
        String sql =
                "update message set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", menderId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    /**
     * 获取消息数量
     *
     * @param params 查询参数
     * @return
     * @throws Exception
     */
    public int count(List<Param> params, RequestContext.Catalog catalog) throws DataBaseException {
        String sql = "select count(1) from message a where a.is_deleted = 0 ";
        if (catalog.equals(RequestContext.Catalog.Tenant)) {
            sql += " and catalog = 'Tenant' ";
        } else {
            sql += " and (catalog = 'Manager' or catalog = 'User') ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 获取消息详情
     *
     * @param id 消息id
     * @return
     * @throws Exception
     */
    public Message query(Long id) throws DataBaseException {
        String sql =
                "select a.*,b.name sender_name from message a left join users b on a.sender = b.id where a.id = :id";
        Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
        List<Message> list = this.list(Message.class, sql, param);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 更新消息状态
     *
     * @param id       消息id
     * @param status   消息状态
     * @param menderId
     * @return
     * @throws Exception
     */
    public boolean updateStatus(Long id, String status, Long menderId) throws DataBaseException {
        String sql =
                "update message set status = :status , gmt_modify = :gmtModify ,mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("menderId", menderId);
        params.put("gmtModify", new Date());
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    /**
     * 消息状态改为已读
     *
     * @param ownerId 消息所属用户id
     * @param catalog 消息所属用户类型
     * @return
     * @throws Exception
     */
    public boolean messageRead(Long ownerId, RequestContext.Catalog catalog) throws DataBaseException {
        String sql;
        Map<String, Object> params = MapTools.simpleMap("ownerId", ownerId);
        if (catalog.equals(RequestContext.Catalog.Tenant)) {
            sql = "update message set status ='READ' where owner_id= :ownerId and catalog = :catalog";
            params.put("catalog", catalog.toString());
        } else {
            sql =
                    "update message set status ='READ' where owner_id= :ownerId  and (catalog = 'Manager' or catalog = 'User') ";
        }

        return this.execute(sql, params) > 0;
    }

    /**
     * 获取不同状态消息数量
     *
     * @param ownerId 消息所属者id
     * @param catalog 消息所属用户类型
     * @return
     * @throws Exception
     */
    public Message countStatus(Long ownerId, RequestContext.Catalog catalog) throws DataBaseException {
        String sql;
        Map<String, Object> params = MapTools.simpleMap("ownerId", ownerId);
        if (catalog.equals(RequestContext.Catalog.Tenant)) {
            sql =
                    "select ifnull(sum(if(status='READ',1,0)),0) as read_count,ifnull(sum(if(status='UNREAD',1,0)),0) as unread_count,count(*) as total from message a where a.owner_id= :ownerId and a.catalog =:catalog ";
            params.put("catalog", catalog.toString());
        } else {
            sql =
                    "select ifnull(sum(if(status='READ',1,0)),0) as read_count,ifnull(sum(if(status='UNREAD',1,0)),0) as unread_count,count(*) as total from message a where a.owner_id= :ownerId and (a.catalog = 'Manager' or a.catalog = 'User')";
        }
        // List<Message> list = this.list(Message.class, sql, params);
        // 适配达梦
        List<Map<String, Object>> list = this.list(sql, params);
        if (list.isEmpty()) {
            Message message = new Message();
            message.setTotal(0L);
            message.setUnreadCount(new BigDecimal("0"));
            message.setReadCount(new BigDecimal("0"));
            return null;
        } else {

            Map<String, Object> result = list.get(0);
            Message message = new Message();
            message.setTotal(Long.valueOf(result.get("total").toString()));
            message.setReadCount(new BigDecimal(result.get("read_count").toString()));
            message.setUnreadCount(new BigDecimal(result.get("unread_count").toString()));
            return message;
        }
        // return list.isEmpty() ? null : list.get(0);
    }
}
